library Intercept42;
uses
  ComServ, ComObj, SConnect, SysUtils, Classes;

type
  TSpyClient42 = class(TComObject, IDataIntercept)
  public
    procedure DataIn(const Data: IDataBlock); stdcall;
    procedure DataOut(const Data: IDataBlock); stdcall;
  end;
  TSpyServer42 = class(TSpyClient42); { use ClassName to differentiate }

const
  SpyClient42_GUID: TGUID = '{DCED4111-4268-4726-87C0-C4FC45592286}';
  SpyServer42_GUID: TGUID = '{8D19590B-C33B-4C93-BCE9-10595FBAD5E1}';

const
  SizeOfHeader = 8;
  Client2Server = $42;
  Server2Client = $66;
  Logfile = 'c:\intercpt.log';

{ TSpyClient42 }

procedure TSpyClient42.DataIn(const Data: IDataBlock);
{ report, decompress or decrypt }
var
  InStream,OutStream: TMemoryStream;
  Size,i: Integer;
  f: TextFile;
  X: Byte;
begin
  // Reporting
  AssignFile(f,LogFile);
  Append(f);
  writeln(f);
  writeln(f,ClassName,' DataIn at: ',DateTimeToStr(Now));
  writeln(f,'Signature: ', Data.Signature);
  writeln(f,'DataSize: ', Data.Size);
  CloseFile(f);
  // Decrypting
  if Data.Size > SizeOfHeader then
  begin
    InStream := TMemoryStream.Create;
    try
      { Skip BytesReserved bytes of data }
      InStream.Write(Pointer(Integer(Data.Memory) + Data.BytesReserved)^, Data.Size);
      Size := InStream.Size;
      if Size = 0 then Exit;
      if Pos('Server',ClassName) > 0 then X := Client2Server
                                     else X := Server2Client;
      OutStream := TMemoryStream.Create;
      try
        OutStream.CopyFrom(InStream, 0);
        for i:=SizeOfHeader to Pred(OutStream.Size) do
          PChar(OutStream.Memory)[i] :=
            Char(Ord(PChar(OutStream.Memory)[i]) XOR X);
      { Clear the datablock, write the encrypte data into the datablock }
        Data.Clear;
        Data.Write(OutStream.Memory^, OutStream.Size)
      finally
        OutStream.Free
      end
    finally
      InStream.Free
    end
  end
end;

procedure TSpyClient42.DataOut(const Data: IDataBlock);
{ report, compress or encrypt }
var
  InStream,OutStream: TMemoryStream;
  Size,i: Integer;
  f: TextFile;
  X: Byte;
begin
  // Reporting
  AssignFile(f,LogFile);
  Append(f);
  writeln(f);
  writeln(f,ClassName,' DataOut at: ',DateTimeToStr(Now));
  writeln(f,'Signature: ', Data.Signature);
  writeln(f,'DataSize: ', Data.Size);
  // Encrypting
  if Data.Size > SizeOfHeader then
  begin
    InStream := TMemoryStream.Create;
    try
      { Skip BytesReserved bytes of data }
      InStream.Write(Pointer(Integer(Data.Memory) + Data.BytesReserved)^, Data.Size);
      Size := InStream.Size;
      if Size = 0 then Exit;
      if Pos('Client',ClassName) > 0 then X := Client2Server
                                     else X := Server2Client;
      OutStream := TMemoryStream.Create;
      try
        OutStream.CopyFrom(InStream, 0);
        for i:=SizeOfHeader to Pred(OutStream.Size) do
        begin
          if PChar(OutStream.Memory)[i] in [#32..#127] then
            write(f,PChar(OutStream.Memory)[i]);
          PChar(OutStream.Memory)[i] :=
            Char(Ord(PChar(OutStream.Memory)[i]) XOR X)
        end;
        writeln(f);
      { Clear the datablock, write the encrypte data into the datablock }
        Data.Clear;
        Data.Write(OutStream.Memory^, OutStream.Size);
      finally
        OutStream.Free
      end
    finally
      InStream.Free
    end
  end;
  CloseFile(f)
end;

{$R *.RES}

exports
  DllGetClassObject,
  DllCanUnloadNow,
  DllRegisterServer,
  DllUnregisterServer;

begin
{ Use this class factory to allow for easy identification of Interceptors }
  TPacketInterceptFactory.Create(ComServer, TSpyClient42, SpyClient42_GUID,
    'SpyClient42', 'Intercept42', ciMultiInstance, tmApartment);
  TPacketInterceptFactory.Create(ComServer, TSpyServer42, SpyServer42_GUID,
    'SpyServer42', 'Intercept42', ciMultiInstance, tmApartment);
end.

